
/*************************************************
****       VEGA - Localization routines       ****
**** Copyright 1996-2002, Alessandro Pedretti ****
*************************************************/

/*
 * The following routines use the AmigaOS Locale Technology to translate the
 * strings in any language using a specific catalog file in standard IFF format.
 * Each locale file must be stored in a sub-directory with the lower-case language
 * name of the Locale directory and must have VEGA.catalog file name.
 * To create a new catalog, you must use FlexCat compiler with the catalog
 * description file (.cd) and the catalog translation file (.ct). Please read the
 * specific documentation in FlexCat directory and in VEGA manual.
 * The AmigaOS Locale Technology is copyrighted by Amiga Inc.
 */


#ifdef __WIN32__
#  include <windows.h>
#endif

#include <stdio.h>
#include <string.h>
#include <stdarg.h>

#ifdef __BORLANDC__
#  pragma hdrstop
#endif

#include "globdef.h"
#include "globvar.h"
#define  VEGA_ARRAY
#include "globstr.h"
#define  __VG_ERRTRANSTAB
#include "comerr.h"
#include "../LocaleLib/locale.h"


#ifdef AMIGA
#  include <exec/libraries.h>
#  include <libraries/locale.h>

extern struct Library  *LocaleBase;
#endif

#ifdef WIN32
#include "gl_global.h"
#include "winutils.h"

extern HWND             hWnd;
static char             LocBuf[4096];

int     ConPrintf(FILE *FH, char *, va_list);
#endif


/**** Shows an error ****/

VG_BOOL CatErr(VG_ULONG Err, ...)
{
  char                  *Ptr;
  va_list               vl;

  SetGlobErr(Err);
  va_start(vl, Err);
  vsprintf(ErrStr, GetStr(Err), vl);
  va_end(vl);

  switch (ErrMode) {
  case VG_ERRMOD_WINDOW:
#ifdef WIN32
    if (GLOBSW_OPENGL) {
      sprintf(LocBuf, GetStr(MSG_ERR_LOCALE_ERRCODE), GlobErr, ErrStr);
      GL_MessageBox(LocBuf, GetStr(MSG_ERR_LOCALE_ERROR), MB_OK|MB_ICONERROR);
      break;
    }
#endif

  case VG_ERRMOD_CONSOLE:
    LocPrintf(stderr, ERROR_PROMPT, GetStr(MSG_ERR_LOCALE_ERROR), GlobErr, ErrStr);
    break;
  } /* End of switch */

  return FALSE;
}


/**** fprintf replacement with string localization ****/

int CatPrintf(FILE *FH, VG_ULONG Str, ...)
{
  int           Ret;
  va_list       vl;

  va_start(vl, Str);

#ifdef WIN32
  Ret = ConPrintf(FH, GetStr(Str), vl);
#else
  Ret = vfprintf(FH, GetStr(Str), vl);
#endif

  va_end(vl);

  return Ret;
}


/**** fprintf replacement with iso2dos character conversion ****/

#ifdef WIN32
int DosPrintf(FILE *FH, char *Str, ...)
{
  va_list       vl;
  int		Ret;

  va_start(vl, Str);
  Ret = ConPrintf(stdout, Str, vl);
  va_end(vl);

  return Ret;
}
#endif


/**** Return a string and if possible, translate it ****/

char *GetStr(VG_ULONG Id)
{
  char                  *BuiltIn = "Unknown localized string";
  register VG_ULONG     k        = sizeof(VEGA_Str) / sizeof(LOCARRAY);
  register LOCARRAY     *Ptr     = (LOCARRAY *)VEGA_Str;

  while(k--) {
    if (Ptr -> Id == Id) {
      BuiltIn = Ptr -> Str;
      break;
    }
    ++Ptr;
  }

  /**** String localization ****/

  if (Catalog)
#ifdef AMIGA
    return((char *)GetCatalogStr((void *)Catalog, Id, BuiltIn));
#else
    return((char *)LocGetCatStr((void *)Catalog, Id, BuiltIn));
#endif

  return BuiltIn;
}


/**** Sets the global error variable ****/

void SetGlobErr(VG_ULONG Err)
{
  register struct _ErrTab       *ErrPtr;

  for(ErrPtr = (struct _ErrTab *)ErrTab;
      (ErrPtr -> ErrCode != VG_PARET_UNKERR) && (ErrPtr -> StrId != (VG_UWORD)Err);
      ++ErrPtr);
  GlobErr = ErrPtr -> ErrCode;
}


#ifdef WIN32

/**** Reset global error ****/

void ResetGlobErr(void)
{
  GlobErr = VG_PARET_OK;
  *ErrStr = 0;
}


/**** Show the last error ****/

void ShowLastErr(void)
{
  if (GlobErr == VG_PARET_OK)
    strcpy(ErrStr, GetStr(MSG_ERR_LOCALE_NOERROR));

  switch (ErrMode) {
  case VG_ERRMOD_WINDOW:
    if (GLOBSW_OPENGL) {
      sprintf(LocBuf, GetStr(MSG_ERR_LOCALE_ERRCODE), GlobErr, ErrStr);
      GL_MessageBox(LocBuf, GetStr(MSG_ERR_LOCALE_LASTERR), MB_OK|MB_ICONERROR);
      break;
    }

  case VG_ERRMOD_CONSOLE:
    LocPrintf(stderr, ERROR_PROMPT, GetStr(MSG_ERR_LOCALE_LASTERR), GlobErr, ErrStr);
    break;
  } /* End of switch */
}

/**** Show a global system error ****/

VG_BOOL WinErr(void)
{
//  char          *lpMsgBuf;

  GlobErr = VG_PARET_WINDOWS;

//  FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM,
  FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM,
                NULL,
                GetLastError(),
                MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
                ErrStr, 255,
//                (LPTSTR)&lpMsgBuf, 0,
                NULL);

  switch (ErrMode) {
  case VG_ERRMOD_WINDOW:
    if (GLOBSW_OPENGL) {
      sprintf(LocBuf, GetStr(MSG_ERR_LOCALE_ERRCODE), GlobErr, ErrStr);
      GL_MessageBox(LocBuf, GetStr(MSG_ERR_LOCALE_ERROR), MB_OK|MB_ICONERROR);
      break;
    }

  case VG_ERRMOD_CONSOLE:
    LocPrintf(stderr, ERROR_PROMPT, GetStr(MSG_ERR_LOCALE_ERROR), GlobErr, ErrStr);
    break;
  } /* End of switch */

//  LocalFree(lpMsgBuf);

  return FALSE;
}
#endif



