
/*************************************************
****      VEGA - Utilities & Subroutines      ****
**** Copyright 1996-2002, Alessandro Pedretti ****
*************************************************/


#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <math.h>
#include <malloc.h>

#include "globdef.h"
#include "globvar.h"
#include "globstr.h"
#include "formats.h"
#include "../LocaleLib/locale.h"

#ifdef AMIGA
#  include <dos/dos.h>
#  include <exec/types.h>
#  include <exec/memory.h>
#  include <exec/execbase.h>
#endif


/**** Add the packer file extension ****/

void AddPackExt(register char *FileName, VG_UWORD PackMode)
{
  register char          **Ptr;
  register VG_ULONG      k;

  if (PackMode) {
    for(Ptr = (char **)(PackExt + 1); *Ptr; ++Ptr) {
      k = strlen(FileName) - strlen(*Ptr);
      if ((FileName[k - 1] == '.') &&
          (!strcasecmp(FileName + k, *Ptr))) break;
    }
    if (!*Ptr) {
      strcat(FileName, ".");
      strcat(FileName, PackExt[PackMode]);
    }
  }
}


/**** Add a value to the radius matrix ****/

void AddToRad(register ATOMO *Atm, register float *Rad, register float Val)
{
  while(Atm) {
    *Rad += Val;
    Atm = Atm -> Ptr;
    ++Rad;
  }
}


/**** Allocates the description structure for one atom  ****/
/**** using the S.O. specific memory allocation routines ****/

ATOMO *AllocAtm(ATOMO **InizAtm, VG_ULONG *TotAtomi)
{
  register ATOMO   *Atm;
  static ATOMO     *PrecAtm;

  if ((Atm = (ATOMO *)SOALLOC(sizeof(ATOMO)))) {
    if (!*TotAtomi) *InizAtm = Atm;
    else PrecAtm -> Ptr = Atm;
    ++*TotAtomi;
    PrecAtm           = Atm;                 /* Bind the list         */
    Atm -> ResName.L  = *(VG_LONG *)"UNK\0"; /* Unknown residue name  */
    *Atm  -> ResSeq.C = '1';                 /* Residue one           */
    Atm  -> ChainID   = ' ';                 /* Default chain         */
    Atm  -> Num       = ++AtmLoaded;         /* Squential atom number */
    *Atm -> Elem.C    = '?';                 /* Unknown element       */
    *Atm -> Pot.C     = '?';                 /* Unknown potential     */
    Atm -> Rad        = 1.8;                 /* Atomic radius         */
    Atm -> ColorID    = 2;                   /* White color           */
    Atm -> Active     = 1;                   /* Active                */
  } else {
    if (*TotAtomi) {
      PDBFree(*InizAtm);
      *InizAtm = NULL;
    }
    CatErr(MSG_ERR_UTILS_OUTOFMEM);
  }

  return Atm;
}


/**** Allocate a surface dot ****/

VG_SURFACE *AllocSrfDot(VG_SURFACE **InizSrf, VG_ULONG *TotDots)
{
  register VG_SURFACE  *Dot;
  static VG_SURFACE    *PrecDot;

  if ((Dot = (VG_SURFACE *)SOALLOC(sizeof(VG_SURFACE)))) {
    if (!*TotDots) *InizSrf = Dot;
    else PrecDot -> Ptr = Dot;
    ++*TotDots;
    PrecDot           = Dot;                 /* Bind the list           */
    Dot -> AtmNum     = 0;                   /* Atom number             */
    Dot -> Val        = 0.0;                 /* Default property value  */
    Dot -> Color[0]   = 0;                   /* Default dot color       */
    Dot -> Color[1]   = 0;
    Dot -> Color[2]   = 255;
    Dot -> Ptr        = NULL;                /* Pointer to next element */
  } else {
    if (*TotDots) {
      SrfFree(*InizSrf);
      *InizSrf = NULL;
    }
    CatErr(MSG_ERR_UTILS_OUTOFMEM);
  }

  return Dot;
}


/**** Calc the geometry center of a molecule ****/

void CalcGeoCent(register ATOMO *Atm, register XYZ *Vect, VG_BOOL GlTrans)
{
  float                 Tmp;

  float                 MaxDist = 0.0;
  VG_ULONG              Tot     = 0;

  if (Atm) {
    Vect -> x = 0.0;
    Vect -> y = 0.0;
    Vect -> z = 0.0;

    while(Atm) {
      Vect -> x += Atm -> x;
      Vect -> y += Atm -> y;
      Vect -> z += Atm -> z;
      if (GlTrans) {
        Tmp = fabs(Atm -> x);
        if (Tmp > MaxDist) MaxDist = Tmp;
        Tmp = fabs(Atm -> y);
        if (Tmp > MaxDist) MaxDist = Tmp;
        Tmp = fabs(Atm -> z);
        if (Tmp > MaxDist) MaxDist = Tmp;
      }
      Atm = Atm -> Ptr;
      ++Tot;
    } /* End of while */

    Vect -> x /= (float)Tot;
    Vect -> y /= (float)Tot;
    Vect -> z /= (float)Tot;

    if (GlTrans) {
      Vect -> x *= -1.0;
      Vect -> y *= -1.0;
      Vect -> z  = Vect -> z * -1.0 - MaxDist * 2.1;
    }
  }
}


/**** Change or add the file extension ****/

void ChangeExt(char *Str, char *Ext)
{
  register char *Ptr;

  for(Ptr = Str + strlen(Str) - 1; Ptr != Str; --Ptr)
    if (*Ptr == '.') {
      strcpy(Ptr + 1, Ext);
      break;
    }
  if ((Ptr == Str) && (*Ptr != '.')) {
    strcat(Ptr, ".");
    strcat(Ptr, Ext);
  }
}


/**** Convert a string into an integer ****/

VG_ULONG Char2Int(char *Str, VG_ULONG k)
{
  VG_ULONG              Molt = 1;
  VG_ULONG              Num  = 0;

  --k;
  do {
    if ((Str[k] >= '0') && (Str[k] <= '9')) {
      Num  += ((Str[k] - '0') * Molt);
      Molt *= 10;
    }
  } while(k--);

  return Num;
}


/**** Check if the force field is assigned ****/

VG_BOOL ChkForceField(register ATOMO *Atm, VG_ULONG TotAtm)
{
  register VG_ULONG  Unassigned = 0;

  do {
    if (Atm -> Pot.L == 0x3f000000) ++Unassigned;
    Atm = Atm -> Ptr;
  } while(Atm);
  if (Unassigned == TotAtm) return FALSE;
  else return TRUE;
}


/**** Close all molecules ****/

void CloseAllMol(void)
{
  CloseSrf();
  SelRemAll();
  PDBFree(BegAtm);
  BegAtm    = NULL;
  LastAtm   = NULL;
  TotalAtm  = 0;
  AtmLoaded = 0;
}


/**** Close the surface ****/

void CloseSrf(void)
{
  SrfFree(BegSrf);
  BegSrf   = NULL;
  TotalSrf = 0;
}


/**** Count bonds ****/

VG_ULONG CountBond(register ATOMO *Atm)
{
  register VG_UWORD  k;
  register VG_ULONG  Count = 0;

  while(Atm) {
    for(k = 0; k < (VG_UWORD)Atm -> NSost; ++k) {
      if (Atm -> Conn[k] -> Num > Atm -> Num)
        ++Count;
    } /* End of for */
    Atm = Atm -> Ptr;
  } /* End of for */

  return Count;
}


/**** Counts the residues ****/

VG_ULONG CountRes(register ATOMO *Atm)
{
  register VG_ULONG  PrecResName = 0;
  register VG_ULONG  PrecResSeq  = 0;
  register VG_ULONG  TotRes      = 0;

  while(Atm) {
    if ((Atm -> ResName.L != (VG_LONG)PrecResName) ||
        (Atm -> ResSeq.L  != (VG_LONG)PrecResSeq)) {
      PrecResName = Atm -> ResName.L;
      PrecResSeq  = Atm -> ResSeq.L;
      ++TotRes;
    }
    Atm = Atm -> Ptr;
  }

  return TotRes;
}


/**** Find a residue ****/

VG_ULONG FindResPtr(register ATOMO *Atm, ATOMO **ResPtr,
                    VG_LONG NomeRes, VG_LONG NRes)
{
  register VG_UWORD  Chk;
  register VG_ULONG  NAtm = 0;

  *ResPtr = 0;

  while(Atm) {
    Chk = 0;
    if (!NomeRes) Chk = 1;
    else if (Atm -> ResName.L == NomeRes) Chk = 1;
    if (!NRes) ++Chk;
    else if (Atm -> ResSeq.L == NRes) ++Chk;
    if (Chk == 2) {
      if (!*ResPtr) *ResPtr = Atm;
      ++NAtm;
    }
    Atm = Atm -> Ptr;
  }

  return NAtm;
}


/**** Reads a record in Fortran format ****/

VG_BOOL ForRead(FILE *FH, void *Ptr, VG_ULONG Len)
{
  VG_BOOL            Ret = TRUE;

  fseek(FH, sizeof(int), SEEK_CUR);
  if (fread(Ptr, Len, 1, FH) == 1)
    fseek(FH, sizeof(int), SEEK_CUR);
  else Ret = PrintDosErr();

  return Ret;
}


/**** Performs Seek from/to Fortran records ****/

VG_BOOL ForSeek(FILE *FH, VG_ULONG Rec, VG_BOOL SwapEndian)
{
  unsigned int  Len;
  VG_BOOL    Ret = TRUE;

  do {
    if (fread(&Len, sizeof(int), 1, FH)) {
      if (SwapEndian)
        Swap(&Len);
      fseek(FH, Len + sizeof(int), SEEK_CUR);
    } else Ret = PrintDosErr();
    --Rec;
  } while ((Rec) && (Ret));

  return Ret;
}


/**** Copy a string from Fortran to C ****/

void ForStrCpy(register char *Dest, register char *Src, VG_ULONG Len)
{
  register VG_ULONG  k;

  for(k = 0;(k < Len) && (Src[k] != ' ');++k)
    Dest[k] = Src[k];
}


/**** Writes a record in Fortran unformatted format ****/

VG_BOOL ForWrite(FILE *FH, void *Num, VG_ULONG Size)
{
  VG_BOOL      Ret = TRUE;

#ifdef LITTLE_ENDIAN
  VG_ULONG     Sz  = SwapI(Size);
#else
  VG_ULONG     Sz  = Size;
#endif

  if ((fwrite(&Sz, sizeof(VG_ULONG), 1, FH) != 1) ||
      (fwrite(Num, Size, 1, FH) != 1) ||
      (fwrite(&Sz, sizeof(VG_ULONG), 1, FH) != 1))
    Ret = PrintDosErr();

  return Ret;
}


/**** Change a string to lowercase ****/

void Lower(register char *Str)
{
  do *Str = (char)tolower(*Str); while(*++Str);
}


/**** Normalize an angle into 0-360 range ****/

double NormAngle(double Ang)
{
  if (Ang < (double)0.0)
    return (360.0 + Ang);

  return Ang;
}


/**** Free a memory list of atoms ****/

void PDBFree(register ATOMO *Atm)
{
  register ATOMO	*Succ;

  if (Atm) {
    do {
      Succ = Atm -> Ptr;
      SOFREE(Atm);
      Atm = Succ;
    } while(Atm);
  }
}


/**** Print an error with atom ****/

VG_BOOL PrintAtmErr(char *Err, ATOMO *Atm, VG_BOOL Force)
{
  sprintf(ErrStr, "%s: %-7d %-4.4s %-2.2s %.4s_%.4s", Err,
          Atm -> Num, Atm -> Name.C, Atm -> Elem.C, Atm -> ResName.C,
          Atm -> ResSeq.C);

  if ((ErrMode != VG_ERRMOD_QUIET) && ((!GLOBSW_STDOUT) || (Force))) {
    LocPrintf(stderr, "%s\n", ErrStr);
  }

  return FALSE;
}


/**** Print loading format ****/

void PrintLoad(VG_ULONG Msg, VG_ULONG NMol)
{
  if ((!GLOBSW_STDOUT) && (NMol))
    CatPrintf(stdout, MSG_UTILS_LOADPROG, NMol, GetStr(Msg));
}


/**** Print a progress message ****/

void PrintProg(VG_ULONG Msg)
{
  if (!GLOBSW_STDOUT)
    LocPrintf(stdout, "%s\n", GetStr(Msg));
}


/**** Remove a single atom from list ****/

void RemoveAtm(ATOMO **Atm, ATOMO **PrecAtm, ATOMO **InizAtm)
{
  if (*Atm == *InizAtm) {
    *InizAtm = (*Atm) -> Ptr;
    *PrecAtm = (*Atm) -> Ptr;
    FREE(*Atm);
    *Atm = *PrecAtm;
  } else {
    (*PrecAtm) -> Ptr = (*Atm) -> Ptr;
    (*PrecAtm) -> Flags |= ((*Atm) -> Flags &~ VG_ATMF_HETATM);
/*
    if (((*Atm) -> Flags & VG_ATMF_SEGEND) && ((*PrecAtm) -> Flags <= 1))
      (*PrecAtm) -> Flags += (VG_BYTE)2;
*/
    if (*Atm == LastAtm) LastAtm = *PrecAtm; 
    FREE(*Atm);
    *Atm = (*PrecAtm) -> Ptr;
  }
}


/**** Remove hydrogens ****/

void RemoveH(ATOMO **InizAtm, VG_ULONG *TotAtm)
{
  register ATOMO      *Con;
  register VG_UWORD   k, j;

  ATOMO               *PrecAtm = *InizAtm;
  ATOMO               *Atm     = *InizAtm;
  VG_ULONG            Count    = 1;

  PrintProg(MSG_UTILS_RMHPROG);

  while(Atm) {
#ifdef LITTLE_ENDIAN
    if (Atm -> Elem.S == 0x48) {
#else
    if (Atm -> Elem.S == 0x4800) {
#endif

      if ((Con = Atm -> Conn[0]) != NULL) {
        for(k = 0;(k < MAXBOND) && (Con -> Conn[k]); ++k) {
          if (Con -> Conn[k] == Atm) {
            for(j = k; j < (MAXBOND - 1); ++j) {
              Con -> Conn[j]  = Con -> Conn[j + 1];
              Con -> Order[j] = Con -> Order[j + 1];
            } /* End of for */
            --Con -> NSost;
          }
        } /* End of for */
      }
      --*TotAtm;
      RemoveAtm(&Atm, &PrecAtm, InizAtm);
    } else {
      PrecAtm    = Atm;
      Atm -> Num = Count++;
      Atm        = Atm -> Ptr;
    }
  }
}


/**** Removes water molecules ****/

void RemoveH2O(ATOMO **InizAtm, VG_ULONG *TotAtm)
{
  ATOMO               *Atm     = *InizAtm;
  ATOMO               *PrecAtm = *InizAtm;
  register VG_ULONG      Count    = 1;

  PrintProg(MSG_UTILS_RMWPROG);

  if (RenameWat(*InizAtm)) {
    Atm     = *InizAtm;
    PrecAtm = Atm;
    while(Atm) {
      if ((Atm -> ResName.C[0] ==  'H') && (Atm -> ResName.C[1] ==  'O') &&
          (Atm -> ResName.C[2] ==  'H')) {
        --*TotAtm;
        RemoveAtm(&Atm, &PrecAtm, InizAtm);
      } else {
        PrecAtm    = Atm;
        Atm -> Num = Count++;
        Atm        = Atm -> Ptr;
      }
    }
  }
}


/**** Remove a residue ****/

void RemoveRes(register ATOMO **Atm, ATOMO **PrecAtm, ATOMO **InizAtm,
               register VG_ULONG *TotAtm)
{
  VG_LONG          ResSeq = (*Atm) -> ResSeq.L;

  do {
    RemoveAtm(Atm, PrecAtm, InizAtm);
    --*TotAtm;
  } while((*Atm) && ((*Atm) -> ResSeq.L == ResSeq));
}


/**** Rename a residue ****/

ATOMO *RenameRes(register ATOMO *Atm, VG_LONG Res)
{
  VG_QCHAR          ResName;
  register ATOMO    *PrecAtm = Atm;
  register VG_LONG  ResSeq   = Atm -> ResSeq.L;

  if ((Atm -> ResName.C[3] == '+') || (Atm -> ResName.C[3] == '-'))
    ((char *)&Res)[2] = Atm -> ResName.C[3];

  ResName.L = Atm -> ResName.L;
  while((Atm) && (ResName.L == Atm -> ResName.L) && (ResSeq == Atm -> ResSeq.L)) {
    Atm -> ResName.L = Res;
    PrecAtm = Atm;
    Atm     = Atm -> Ptr;
  }

  return PrecAtm;
}


/**** Rename waters ****/

VG_BOOL RenameWat(register ATOMO *Atm)
{
  register ATOMO        *H1, *H2;

  VG_BOOL		Ret  = FALSE;

  while(Atm) {
    H1 = Atm -> Conn[0];
    H2 = Atm -> Conn[1];

#ifdef LITTLE_ENDIAN
    if ((Atm -> Elem.S == 0x4f) && (Atm -> NSost == 2) &&
        (H1 -> Elem.S == 0x48) && (H2 -> Elem.S == 0x48)) {
      Atm -> ResName.L = 0x484F48;
      H1  -> ResName.L = 0x484F48;
      H2  -> ResName.L = 0x484F48;
#else
    if ((Atm -> Elem.S == 0x4f00) && (Atm -> NSost == 2) &&
        (H1 -> Elem.S == 0x4800) && (H2 -> Elem.S == 0x4800)) {
      Atm -> ResName.L = 0x484F4800;
      H1  -> ResName.L = 0x484F4800;
      H2  -> ResName.L = 0x484F4800;
#endif
      Ret = TRUE;
    }
    Atm = Atm -> Ptr;
  }

  return Ret;
}


/**** Renumber atoms ****/

void RenAtm(register ATOMO *Atm, register VG_ULONG StartNum)
{
  while(Atm) {
    Atm -> Num = StartNum++;
    Atm = Atm -> Ptr;
  } /* End of while */
}


/**** Renumbers residues ****/

void RenRes(register ATOMO *Atm, register VG_ULONG ResSeq)
{
  char      Tmp[6];

  register VG_ULONG  PrecResSeq = Atm -> ResSeq.L;

  PrintProg(MSG_UTILS_RENRESPROG);

  while(Atm) {
    if (Atm -> ResSeq.L != (VG_LONG)PrecResSeq) {
      PrecResSeq = Atm -> ResSeq.L;
      ++ResSeq;
    }
    sprintf(Tmp, "%d", ResSeq);
    Str2Qchar(&Atm -> ResSeq, Tmp);
    Atm = Atm -> Ptr;
  }
}


/**** Check atom selection ****/

VG_BOOL SelAtmChk(ATMLST *Sel)
{
  VG_BOOL               Ret;

  int                   Count = 0;
  register ATOMO        *Atm  = BegAtm;

  while(Atm) {
    if ((Atm -> Name.L == Sel -> Name.L) &&
        ((!Sel -> ResName.L) || (Atm -> ResName.L == Sel -> ResName.L)) &&
        ((!Sel -> ResSeq.L) || (Atm -> ResSeq.L == Sel -> ResSeq.L))) {
      Sel -> Atm = Atm;
      ++Count;
    }
    Atm = Atm -> Ptr;
  }

  if (Count != 1) {
    Sel -> Atm = NULL;
    Ret        = FALSE;
  } else Ret = TRUE;

  return Ret;
}


/**** Remove all selected atoms ****/

void SelAtmRemAll(register ATMSEL *Sel)
{
  register ATMLST       *Rem;
  register ATMLST       *List = Sel -> BegList;

  while(List) {
    Rem  = List;
    List = List -> Ptr;
    FREE(Rem);
  } /* End of while */

  Sel -> BegList = NULL;
  Sel -> LastAtm = NULL;
}


/**** Return the pointer of a selection ****/

ATMSEL *SelRetPtr(VG_WORD Index, VG_WORD Type)
{
  register ATMSEL       *Sel = BegSel;

  while(Sel) {
    if ((Type == SL_NONE) || (Type == Sel -> Type))
      --Index;
    if (Index)
      Sel = Sel -> Ptr;
    else
      break;
  } /* End of while */

  return Sel;
}


/**** Remove all selections ****/

void SelRemAll(void)
{
  register ATMSEL       *NextSel;

  register ATMSEL       *Sel = BegSel;

  while(Sel) {
    SelAtmRemAll(Sel);
    NextSel = Sel -> Ptr;
    FREE(Sel);
    Sel = NextSel;
  } /* End of while */

  BegSel  = NULL;
  LastSel = NULL;
}


/**** Translates a string into a DCHAR ****/

void Str2Dchar(register VG_DCHAR *Res, register char *Str)
{
  (*Res).S    = 0;
  (*Res).C[0] = Str[0];
  if (Str[0])(*Res).C[1] = Str[1];
}


/**** Translates a string into a QCHAR ****/

void Str2Qchar(VG_QCHAR *Res, register char *Str)
{
  register char      *Dest = (*Res).C;
  register VG_UWORD  k;

  (*Res).L = (VG_ULONG)NULL;
  for(k = 0;(*Str) && (k < 4); ++k) {
   *Dest = *Str;
   ++Str;
   ++Dest;
  }
}


/**** Restore the force field parameters ****/

void RestoreFF(register ATOMO *Atm, register VG_LONG *Store)
{
  do {
    Atm -> Pot.L = *Store;
    Atm = Atm -> Ptr;
    ++Store;
  } while (Atm);
}


/**** Set the residue type ****/

void SetTypeRes(VG_QCHAR *Res, VG_UBYTE Car)
{
  if (Res -> L != *(VG_LONG *)"ASP-") {
    if ((Res -> C[3] == '+') || (Res -> C[3] == '-')) {
      Res -> C[2] = Res -> C[3];
    }
  } else Res -> L = *(VG_LONG *)"AP-";

  Res -> C[3] = Car;
}


/**** Free a memory list of surface dots ****/

void SrfFree(register VG_SURFACE *Dot)
{
  register VG_SURFACE	*Succ;

  if (Dot) {
    do {
      Succ = Dot -> Ptr;
      SOFREE(Dot);
      Dot = Succ;
    } while(Dot);
  }
}


/**** Store the force field parameters ****/

VG_LONG *StoreFF(register ATOMO *Atm, VG_ULONG TotAtm)
{
  VG_LONG               *Store;
  register VG_LONG      *Ptr;

  if ((Store = (VG_LONG *)Alloca(TotAtm * sizeof(VG_LONG)))) {
    Ptr = Store;
    do {
      *Ptr = Atm -> Pot.L;
      Atm = Atm -> Ptr;
      ++Ptr;
    } while (Atm);
  }

  return Store;
}


/**** Change the endian (if needed) and writes a long ****/

int SwapWrite(FILE *FH, void *Val)
{
  VG_LONG       Num = *(VG_LONG *)Val;
#ifdef LITTLE_ENDIAN
  Swap(&Num);
#endif

  return (int)fwrite(&Num, sizeof(VG_ULONG), 1, FH);
}


/**** Change a string to uppercase ****/

void Upper(register char *Str)
{
  do *Str = (char)toupper(*Str); while(*++Str);
}


/**** Input prompt for a string ****/

void VegaGetString(char *Prompt, char *Str, VG_UWORD Len)
{
  printf("%s: ", Prompt);
  fgets(Str, Len, stdin);
  Str[strlen(Str) - 1] = 0;
}


/**** Wildcard compare ****/

int Wildncmp(register char *Wild, register char *String, int Len)
{
  register char         *Cp, *Mp;

  register char         *End = String + Len;

  while ((String != End) && (*String) && (*Wild != '*')) {
    if ((*Wild != *String) && (*Wild != '?')) {
      return 0;
    }
    Wild++;
    String++;
  } /* End of while */

  while ((String != End) && (*String)) {
    if (*Wild == '*') {
      if (!*++Wild) return 1;
      Mp = Wild;
      Cp = String + 1;
    } else if ((*Wild == *String) || (*Wild == '?')) {
      Wild++;
      String++;
    } else {
      Wild   = Mp;
      String = Cp++;
    }
  } /* End of while */

  while (*Wild == '*') Wild++;

  return !*Wild;
}

